import django
from django.conf import settings as django_settings
from django.utils.functional import LazyObject

from comment import settings as app_settings


class LazySettings(LazyObject):
    def _setup(self):
        self._wrapped = Settings(getattr(app_settings, 'COMMENT_SETTINGS', {}), getattr(django_settings, 'COMMENT_SETTINGS', {}))


class Settings(object):
    DEPRECATED_SETTINGS = {
        'PASSWORD_RESET_TIMEOUT_DAYS' if django.VERSION > (3, 0) else None,
        'DEFAULT_CONTENT_TYPE' if django.VERSION > (2, 2) else None,
        'FILE_CHARSET' if django.VERSION > (2, 2) else None,
        'USE_L10N' if django.VERSION > (4, 0) else None,
        'USE_TZ' if django.VERSION > (4, 0) else None,
    }

    def __init__(self, *args):
        for item in args:
            for attr in item.keys():
                setattr(self, attr, item.get(attr))


settings = LazySettings()



# Generated by Django 4.1.7 on 2023-04-06 05:42

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.TextField()),
                ('is_spoiler', models.BooleanField(default=False)),
                ('is_pinned', models.BooleanField(default=False)),
                ('status', models.CharField(choices=[('d', 'Delivered'), ('a', 'Accepted'), ('r', 'Rejected')], default='d', max_length=1)),
                ('urlhash', models.CharField(editable=False, max_length=50, unique=True)),
                ('posted', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='comment.comment')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comments', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('-is_pinned', '-posted'),
            },
        ),
        migrations.CreateModel(
            name='CommentSettings',
            fields=[
                ('name', models.CharField(max_length=30)),
                ('slug', models.SlugField(help_text='This value will be used in render_comments tag', primary_key=True, serialize=False)),
                ('default_profile_image', models.ImageField(blank=True, null=True, upload_to='comment_default_profile_images')),
                ('content_words_count', models.PositiveSmallIntegerField(default=40, help_text='More than this value will have Read More button in comment content')),
                ('status_check', models.BooleanField(default=False, help_text='If True, comment status will be set as d(Delivered) otherwise it will be set as a(Accepted).')),
                ('allow_spoiler', models.BooleanField(default=True)),
                ('allow_reply', models.BooleanField(default=True)),
                ('allow_edit', models.BooleanField(default=True)),
                ('allow_delete', models.BooleanField(default=True)),
                ('allow_reaction', models.BooleanField(default=True, help_text='First, create a react emoji in React models')),
                ('reaction_type', models.CharField(choices=[('emoji', 'Emoji'), ('source', 'Source')], default='emoji', help_text='Add source in React model', max_length=6)),
                ('per_page', models.PositiveSmallIntegerField(default=10, help_text="Set 0 if you don't want pagination (All comments will be shown at once)")),
                ('time_type', models.PositiveSmallIntegerField(choices=[(1, 'Compound'), (2, 'From Now'), (3, 'Date & Time')], default=1, help_text='Comment posted time style')),
                ('time_days', models.PositiveSmallIntegerField(default=3, help_text='Less than this value will use FROM NOW type , more will use DATE & TIME type')),
                ('theme_direction', models.CharField(choices=[('ltr', 'LTR (Left to Right)'), ('rtl', 'RTL (Right to Left)')], default='ltr', max_length=3)),
                ('theme_dark_mode', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name': 'Comment Settings',
                'verbose_name_plural': 'Comment Settings',
            },
        ),
        migrations.CreateModel(
            name='React',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.CharField(max_length=20, unique=True)),
                ('emoji', models.CharField(max_length=5)),
                ('source', models.ImageField(blank=True, null=True, upload_to='react_source')),
            ],
        ),
        migrations.CreateModel(
            name='Reaction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('comment', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='reactions', to='comment.comment')),
                ('react', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='reactions', to='comment.react')),
                ('user', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='comment_reactions', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'unique_together': {('user', 'comment')},
            },
        ),
    ]



from django.contrib.auth import get_user_model
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.html import format_html

from comment.managers import CommentQuerySet, ReactionQuerySet

User = get_user_model()


class CommentSettings(models.Model):
    name = models.CharField(max_length=30)
    slug = models.SlugField(primary_key=True, help_text='This value will be used in render_comments tag')

    default_profile_image = models.ImageField(upload_to='comment_default_profile_images', null=True, blank=True)
    content_words_count = models.PositiveSmallIntegerField(default=40, help_text='More than this value will have Read More button in comment content')
    status_check = models.BooleanField(default=False, help_text='If True, comment status will be set as d(Delivered) otherwise it will be set as a(Accepted).')
    status_edited_check = models.BooleanField(default=False, help_text='If True, comment status_edited will be set as d(Delivered) otherwise it will be set as a(Accepted) when a comment is edited.')
    allow_spoiler = models.BooleanField(default=True)
    allow_reply = models.BooleanField(default=True)
    allow_edit = models.BooleanField(default=True)
    allow_delete = models.BooleanField(default=True)
    allow_reaction = models.BooleanField(default=True, help_text='First, create a react emoji in React models')
    reaction_type = models.CharField(max_length=6, choices=(('emoji', 'Emoji'), ('source', 'Source')), default='emoji', help_text='Add source in React model')
    per_page = models.PositiveSmallIntegerField(default=10, help_text='Set 0 if you don\'t want pagination (All comments will be shown at once)')
    time_type = models.PositiveSmallIntegerField(default=1, choices=((1, 'Compound'), (2, 'From Now'), (3, 'Date & Time')), help_text='Comment posted time style')
    time_days = models.PositiveSmallIntegerField(default=3, help_text='Less than this value will use FROM NOW type , more will use DATE & TIME type')
    theme_direction = models.CharField(max_length=3, choices=(('ltr', 'LTR (Left to Right)'), ('rtl', 'RTL (Right to Left)')), default='ltr')
    theme_dark_mode = models.BooleanField(default=False)

    class Meta:
        verbose_name = 'Comment Settings'
        verbose_name_plural = 'Comment Settings'

    def __str__(self):
        return f'{self.name} - [{self.slug}]'


class Comment(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='comments')
    content_main = models.TextField()
    content = models.TextField()  # edited content will store here
    parent = models.ForeignKey('self', on_delete=models.CASCADE, related_name='children', null=True, blank=True)
    is_spoiler = models.BooleanField(default=False)
    is_pinned = models.BooleanField(default=False)
    STATUS_CHOICES = (('d', 'Delivered'), ('a', 'Accepted'), ('r', 'Rejected'))
    status = models.CharField(max_length=1, choices=STATUS_CHOICES, default='d')
    STATUS_EDITED_CHOICES = (('n', 'None'), ('d', 'Delivered'), ('a', 'Accepted'), ('r', 'Rejected'))
    status_edited = models.CharField(max_length=1, choices=STATUS_EDITED_CHOICES, default='n')
    urlhash = models.CharField(max_length=50, unique=True, editable=False)
    posted = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    objects = CommentQuerySet.as_manager()

    class Meta:
        ordering = ('-is_pinned', '-posted')

    def __str__(self):
        if not self.parent:
            return f'{self.content_main[:20]}'
        else:
            return f'[RE] ({self.parent.content_main[:10]}) : {self.content_main[:15]}'

    def set_unique_urlhash(self):
        if not self.urlhash:
            self.urlhash = self.__class__.objects.generate_urlhash()
            while self.__class__.objects.filter(urlhash=self.urlhash).exists():
                self.urlhash = self.__class__.objects.generate_urlhash()

    def save(self, *args, **kwargs):
        self.set_unique_urlhash()
        super(Comment, self).save(*args, **kwargs)

    def is_updated(self):
        return True if self.updated.timestamp() > self.posted.timestamp() and self.status_edited == 'a' else False

    is_updated.boolean = True

    @property
    def is_parent(self):
        return self.parent is None


class React(models.Model):
    slug = models.CharField(max_length=20, unique=True)
    emoji = models.CharField(max_length=5)
    source = models.ImageField(upload_to='react_source', null=True, blank=True)

    def __str__(self):
        return self.slug

    def source_file(self):
        return format_html(f"<img style='height:20px' src='{self.source.url}' alt='{self.emoji}'>") if self.source else 'X'


class Reaction(models.Model):
    user = models.ForeignKey(User, related_name='comment_reactions', on_delete=models.CASCADE, editable=False)
    comment = models.ForeignKey(Comment, related_name='reactions', on_delete=models.CASCADE, editable=False)
    react = models.ForeignKey(React, related_name='reactions', on_delete=models.CASCADE, editable=False)

    objects = ReactionQuerySet.as_manager()

    class Meta:
        unique_together = ['user', 'comment']

    def __str__(self):
        return f'{self.user} <{self.react.slug}> ({self.comment.content_main[:20]})'



from datetime import datetime, timedelta, timezone

from django import template
from django.contrib.contenttypes.models import ContentType
from django.templatetags.static import static

from comment import settings
from comment.models import CommentSettings

register = template.Library()


@register.inclusion_tag('comment/utils/IMPORTS.html')
def render_comment_import():
    return {'offline_imports': settings.OFFLINE_IMPORTS}


@register.inclusion_tag('comment/comment/comments.html')
def render_comments(request, obj, settings_slug):
    context = {
        'object': obj,
        'request': request,
        'settings': CommentSettings.objects.get(slug=settings_slug),
        'object_info': {
            'app_name': type(obj)._meta.app_label,
            'model_name': type(obj).__name__,
            'content_type': ContentType.objects.get_for_model(obj),
            'object_id': obj.id
        }
    }
    return context


@register.simple_tag
def get_settings(settings_parameter):
    return getattr(settings, settings_parameter)


@register.simple_tag
def get_profile_image(user):
    if settings.PROFILE_IMAGE_FIELD:
        profile = getattr(user, settings.PROFILE_IMAGE_FIELD)
        if profile:
            return profile.url
    else:
        return None


@register.filter
def number(value, floating_points=None):
    converters = {
        3: 'K',
        6: 'M',
        9: 'B',
    }

    if floating_points is not None and (10 ** 12 > value >= 1000):
        for exponent, converter in converters.items():
            large_number = 10 ** exponent
            if value >= large_number * 1000:
                continue
            return f'{value / large_number:,.{floating_points}f} {converter}'
    else:
        return f'{value:,}'


@register.simple_tag
def get_pagination(paginator):
    if paginator.paginator.num_pages <= 9:
        return paginator.paginator.page_range
    else:
        if paginator.number <= 5:
            return [*range(1, 7), 0, paginator.paginator.num_pages]
        elif paginator.number > paginator.paginator.num_pages - 5:
            return [1, 0, *range(paginator.paginator.num_pages - 5, paginator.paginator.num_pages + 1)]
        else:
            return [1, 0, paginator.number - 2, paginator.number - 1, paginator.number, paginator.number + 1, paginator.number + 2, 0, paginator.paginator.num_pages]


@register.filter
def is_time_lt_days(time, days):
    return True if datetime.now(tz=timezone.utc) - time < timedelta(days=days) else False



from django.urls import path
from comment import views

app_name = 'comment'
urlpatterns = [
    path('detail/', views.CommentDetail.as_view(), name='detail'),
    path('list/', views.CommentList.as_view(), name='list'),
    path('create/', views.CommentCreate.as_view(), name='create'),
    path('update/', views.CommentUpdate.as_view(), name='update'),
    path('delete/<str:urlhash>/', views.CommentDelete.as_view(), name='delete'),
    path('react/', views.CommentReact.as_view(), name='react'),
]



import io
import os
import time

import requests
from django.core.files import File
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.db import migrations


def save_image_from_url(url):
    response = requests.get(url)
    file = io.BytesIO(response.content)
    filename = os.path.basename(url)
    name, ext = os.path.splitext(filename)
    file_name = f'{name}-{time.time()}{ext}'
    uploaded_file = InMemoryUploadedFile(file, None, file_name, 'image/svg', file.getbuffer().nbytes, None)
    return uploaded_file, file_name


def InitialCommentSettings(apps, schema_editor):
    CommentSettings = apps.get_model('comment', 'CommentSettings')

    print('Fetching default_profile_image from web')

    # STAR RATING CONFIG
    file, file_name = save_image_from_url(url="https://raw.githubusercontent.com/mahyar-amiri/django-comment-system/master/comment/static/comment/img/profile.png")
    CommentSettings.objects.create(name='Default Config', slug='default-config', default_profile_image=File(file, file_name))


def InitialReact(apps, schema_editor):
    React = apps.get_model('comment', 'React')
    React.objects.create(slug='like', emoji='👍')
    React.objects.create(slug='dislike', emoji='👎')


class Migration(migrations.Migration):
    dependencies = [
        ('comment', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(InitialCommentSettings),
        migrations.RunPython(InitialReact),
    ]



from django.conf import settings
from django.urls import reverse_lazy

_COMMENT_SETTINGS = getattr(settings, 'COMMENT_SETTINGS', {})
login_url = getattr(settings, 'LOGIN_URL')

COMMENT_SETTINGS = {
    'LOGIN_URL': reverse_lazy(login_url) if type(login_url) == str else login_url,
    'LANGUAGE_CODE': getattr(settings, 'LANGUAGE_CODE'),
    'URLHASH_LENGTH': _COMMENT_SETTINGS.get('URLHASH_LENGTH', 8),
    'OFFLINE_IMPORTS': _COMMENT_SETTINGS.get('OFFLINE_IMPORTS', True),
    'PROFILE_IMAGE_FIELD': _COMMENT_SETTINGS.get('PROFILE_IMAGE_FIELD', None),
}



from django.core.exceptions import PermissionDenied
from django.contrib.auth.mixins import LoginRequiredMixin


class CommentMixin(LoginRequiredMixin):
    def has_object_permission(self, obj):
        return True if obj.user == self.request.user else False

    def get_object(self, *args, **kwargs):
        obj = super().get_object(*args, **kwargs)
        if self.has_object_permission(obj):
            return obj
        else:
            raise PermissionDenied("Access Denied : update or delete comment.")



from random import choice
from string import ascii_lowercase

from django.db import models

from comment import settings


class CommentQuerySet(models.QuerySet):
    def filter_accepted(self):
        return self.filter(status='a')

    def filter_parents(self):
        return self.filter(parent__isnull=True)

    def filter_updated(self):
        return self.filter(updated__gt=models.F('posted'))

    def filter_not_updated(self):
        return self.filter(updated=models.F('posted'))

    def order_newest(self):
        return self.order_by('-posted')

    def order_pinned_newest(self):
        return self.order_by('-is_pinned', '-posted')

    def order_oldest(self):
        return self.order_by('posted')

    def order_pinned_oldest(self):
        return self.order_by('-is_pinned', 'posted')

    @staticmethod
    def generate_urlhash():
        return ''.join(choice(ascii_lowercase) for _ in range(settings.URLHASH_LENGTH))


class ReactionQuerySet(models.QuerySet):
    def get_reacts(self):
        from comment.models import React

        reacts = React.objects.all()
        return {react: self.filter(react=react) for react in reacts}

    def get_users(self):
        return [reaction.user for reaction in self.all()]



import json

from django.contrib.contenttypes.models import ContentType
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.http import JsonResponse, HttpResponse, HttpResponseBadRequest
from django.shortcuts import render
from django.utils import timezone
from django.views import View
from django.views.generic import ListView, TemplateView

from comment.forms import CommentForm
from comment.mixins import CommentMixin
from comment.models import Comment, Reaction, React, CommentSettings


class CommentDetail(TemplateView):
    template_name = 'comment/comment/comment_body.html'

    def get(self, request, *args, **kwargs):
        urlhash = request.GET.get('urlhash')
        settings_slug = request.GET.get('settings_slug')
        context = {
            'comment': Comment.objects.get(urlhash=urlhash),
            'settings': CommentSettings.objects.get(slug=settings_slug)
        }
        return render(request, self.template_name, context=context)


class CommentList(ListView):
    context_object_name = 'comments'
    template_name = 'comment/comment/comment_list.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        settings_slug = self.request.GET.get('settings_slug')
        context['settings'] = CommentSettings.objects.get(slug=settings_slug)
        return context

    def get_queryset(self):
        queryset = None
        if self.request.GET:
            app_name = self.request.GET.get('app_name')
            model_name = self.request.GET.get('model_name')
            object_id = self.request.GET.get('object_id')
            settings_slug = self.request.GET.get('settings_slug')
            comment_settings = CommentSettings.objects.get(slug=settings_slug)
            content_type = ContentType.objects.get(app_label=app_name, model=model_name.lower())
            queryset = Comment.objects.filter(content_type=content_type, object_id=object_id)
            queryset = queryset.filter_accepted().filter_parents()
            queryset = queryset.order_pinned_newest()

            # Pagination
            page = self.request.GET.get('page', 1)
            if comment_settings.per_page == 0:
                paginator = Paginator(queryset, queryset.count())
            else:
                paginator = Paginator(queryset, comment_settings.per_page)
            try:
                queryset = paginator.page(page)
            except PageNotAnInteger:
                queryset = paginator.page(1)
            except EmptyPage:
                queryset = paginator.page(paginator.num_pages)

        return queryset


class CommentCreate(CommentMixin, View):
    def post(self, request, *args, **kwargs):
        data = json.loads(request.body)
        form = CommentForm(data=data)
        if form.is_valid():
            comment = form.save(commit=False)

            comment.user = request.user
            comment.content_main = comment.content
            app_name = data.get('app_name', None)
            model_name = data.get('model_name', None)
            object_id = data.get('object_id', None)
            parent_id = data.get('parent_id', None)
            settings_slug = data.get('settings_slug', None)
            comment_settings = CommentSettings.objects.get(slug=settings_slug)
            time_posted = timezone.now()
            comment.content_type = ContentType.objects.get(app_label=app_name, model=model_name.lower())
            comment.object_id = object_id

            if parent_id:
                parent_comment = Comment.objects.get(urlhash=parent_id)
                comment.parent = parent_comment
            else:
                comment.parent = None
            comment.posted = time_posted

            if not comment_settings.status_check:
                comment.status = 'a'

            comment.save()
            return HttpResponse(status=200)
        else:
            return HttpResponseBadRequest()


class CommentUpdate(CommentMixin, View):
    def post(self, request, *args, **kwargs):
        data = json.loads(request.body)
        form = CommentForm(data=data)
        if form.is_valid():
            comment = Comment.objects.get(urlhash=data.get('urlhash'))
            new_content = data.get('content')
            new_is_spoiler = data.get('is_spoiler')
            if comment.content_main != new_content or comment.is_spoiler != new_is_spoiler:
                comment.content = new_content
                comment.is_spoiler = new_is_spoiler

                settings_slug = data.get('settings_slug', None)
                comment_settings = CommentSettings.objects.get(slug=settings_slug)

                if comment_settings.status_edited_check and comment.content_main != comment.content:
                    comment.status_edited = 'd'
                    comment.save()
                else:
                    comment.status_edited = 'a'
                    comment.content_main = comment.content
                    comment.save()

                return JsonResponse({'urlhash': comment.urlhash}, status=200)
            return HttpResponse(status=200)
        else:
            return HttpResponseBadRequest()


class CommentDelete(CommentMixin, View):
    def get(self, request, urlhash, *args, **kwargs):
        return render(request, 'comment/forms/comment_form_delete.html', context={'urlhash': urlhash})

    def post(self, request, urlhash, *args, **kwargs):
        comment = Comment.objects.get(urlhash=urlhash)
        if comment:
            comment.delete()
            return HttpResponse(status=200)
        return HttpResponseBadRequest()


class CommentReact(CommentMixin, View):
    def get(self, request, *args, **kwargs):
        urlhash = request.GET.get('urlhash')
        context = {'comment': Comment.objects.get(urlhash=urlhash)}
        return render(request, 'comment/comment/comment_reactions.html', context=context)

    def post(self, request, *args, **kwargs):
        user = request.user
        data = json.loads(request.body)
        comment_urlhash = data.get('urlhash', None)
        react_slug = data.get('react_slug', None)

        reaction = Reaction.objects.filter(user=user, comment__urlhash=comment_urlhash).first()

        if reaction:  # Update Previous Reaction
            if reaction.react.slug == react_slug:  # Delete Previous Reaction
                reaction.delete()
            else:  # Change Previous Reaction
                react = React.objects.get(slug=react_slug)
                reaction.react = react
                reaction.save()
        else:  # Create New Reaction
            comment = Comment.objects.get(urlhash=comment_urlhash)
            react = React.objects.get(slug=react_slug)
            Reaction.objects.create(user=user, comment=comment, react=react)

        return HttpResponse(status=200)



from django.contrib import admin, messages
from django.utils.translation import ngettext

from comment.models import Comment, React, Reaction, CommentSettings


class ListFilterByParent(admin.SimpleListFilter):
    title = 'parent'
    parameter_name = 'parent'

    def lookups(self, request, model_admin):
        return (
            ('parent', 'Comments'), ('child', 'Replies')
        )

    def queryset(self, request, queryset):
        if self.value():
            if self.value() == 'parent':
                return queryset.filter(parent__isnull=True)
            elif self.value() == 'child':
                return queryset.filter(parent__isnull=False)


class ListFilterByUpdated(admin.SimpleListFilter):
    title = 'is updated'
    parameter_name = 'is_updated'

    def lookups(self, request, model_admin):
        return (
            ('updated', 'Yes'), ('not_updated', 'No')
        )

    def queryset(self, request, queryset):
        if self.value():
            if self.value() == 'updated':
                return queryset.filter_updated()
            elif self.value() == 'not_updated':
                return queryset.filter_not_updated()


class CommentAdmin(admin.ModelAdmin):
    list_display = ('__str__', 'content', 'user', 'is_spoiler', 'is_pinned', 'is_updated', 'status', 'status_edited', 'content_object', 'posted', 'updated')
    ordering = ('-is_pinned', '-posted',)
    search_fields = ('content',)
    list_filter = ('is_spoiler', 'is_pinned', ListFilterByUpdated, 'status', 'status_edited', ListFilterByParent)
    readonly_fields = ('user', 'content_main', 'content', 'parent', 'content_type', 'content_object', 'object_id', 'urlhash', 'posted', 'is_spoiler', 'is_pinned', 'status', 'status_edited')
    actions = ['accept_comment', 'reject_comment', 'accept_edited_comment', 'reject_edited_comment', 'pin_comment', 'unpin_comment', 'set_spoiler_comment', 'unset_spoiler_comment']

    def accept_comment(self, request, queryset):
        accepted = queryset.update(status='a')
        self.message_user(request,
                          ngettext(f'{accepted} comment accepted.',
                                   f'{accepted} comments accepted.', accepted),
                          messages.SUCCESS)

    accept_comment.short_description = 'Accept selected comments'

    def reject_comment(self, request, queryset):
        rejected = queryset.update(status='r')
        self.message_user(request,
                          ngettext(f'{rejected} comment rejected.',
                                   f'{rejected} comments rejected.', rejected),
                          messages.WARNING)

    reject_comment.short_description = 'Reject selected comments'

    def accept_edited_comment(self, request, queryset):
        for comment in queryset:
            comment.content_main = comment.content
            comment.save()
        edited = queryset.update(status_edited='a')
        self.message_user(request,
                          ngettext(f'{edited} comment edited.',
                                   f'{edited} comments edited.', edited),
                          messages.SUCCESS)

    accept_edited_comment.short_description = 'Accept edit for selected comments'

    def reject_edited_comment(self, request, queryset):
        for comment in queryset:
            comment.content = comment.content_main
            comment.save()
        edited = queryset.update(status_edited='r')
        self.message_user(request,
                          ngettext(f'{edited} comment did not edit.',
                                   f'{edited} comments did not edit.', edited),
                          messages.WARNING)

    reject_edited_comment.short_description = 'Reject edit for selected comments'

    def pin_comment(self, request, queryset):
        pinned = queryset.update(is_pinned=True)
        self.message_user(request,
                          ngettext(f'{pinned} comment pinned.',
                                   f'{pinned} comments pinned.', pinned),
                          messages.SUCCESS)

    pin_comment.short_description = 'Pin selected comments'

    def unpin_comment(self, request, queryset):
        unpinned = queryset.update(is_pinned=False)
        self.message_user(request,
                          ngettext(f'{unpinned} comment unpinned.',
                                   f'{unpinned} comments unpinned.', unpinned),
                          messages.SUCCESS)

    unpin_comment.short_description = 'Unpin selected comments'

    def set_spoiler_comment(self, request, queryset):
        spoiler = queryset.update(is_spoiler=True)
        self.message_user(request,
                          ngettext(f'{spoiler} comment set as spoiler.',
                                   f'{spoiler} comments set as spoiler.', spoiler),
                          messages.SUCCESS)

    set_spoiler_comment.short_description = 'Set selected comments as spoiler'

    def unset_spoiler_comment(self, request, queryset):
        spoiler = queryset.update(is_spoiler=False)
        self.message_user(request,
                          ngettext(f'{spoiler} comment unset as spoiler.',
                                   f'{spoiler} comments unset as spoiler.', spoiler),
                          messages.SUCCESS)

    unset_spoiler_comment.short_description = 'Unset selected comments as spoiler'


class ReactionAdmin(admin.ModelAdmin):
    list_display = ('user', 'comment', 'react')
    list_filter = ('react__emoji',)
    search_fields = ('user', 'comment')
    readonly_fields = ('user', 'comment', 'react')


class ReactAdmin(admin.ModelAdmin):
    list_display = ('slug', 'emoji', 'source_file')
    search_fields = ('slug', 'emoji')


class SettingsAdmin(admin.ModelAdmin):
    list_display = ('name', 'slug')
    search_fields = ('name', 'slug')
    prepopulated_fields = {'slug': ('name',)}


admin.site.register(CommentSettings, SettingsAdmin)
admin.site.register(Comment, CommentAdmin)
admin.site.register(Reaction, ReactionAdmin)
admin.site.register(React, ReactAdmin)



# Generated by Django 4.1.7 on 2023-04-06 21:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('comment', '0002_config'),
    ]

    operations = [
        migrations.AddField(
            model_name='comment',
            name='content_main',
            field=models.TextField(default=''),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='comment',
            name='status_edited',
            field=models.CharField(choices=[('n', 'None'), ('d', 'Delivered'), ('a', 'Accepted'), ('r', 'Rejected')], default='n', max_length=1),
        ),
        migrations.AddField(
            model_name='commentsettings',
            name='status_edited_check',
            field=models.BooleanField(default=False, help_text='If True, comment status_edited will be set as d(Delivered) otherwise it will be set as a(Accepted) when a comment is edited.'),
        ),
    ]
